-------------------------------------------------------------------------------
-- Title      : toplevel_tb
-- Project    : 
-------------------------------------------------------------------------------
-- File       : toplevel_tb.vhd
-- Author     : Ola Martin Tiseth Stoevneng
-- Company    : 
-- Last update: 2014/02/04
-- Platform   : Spartan-6 LX150T
-------------------------------------------------------------------------------
-- Description: TB for toplevel.
--            : Use along with modelsim commands to set PCI signals
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author   Description
-- 2014/02/04  1.0      stovneng Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
 
use ieee.numeric_std.all;

 
entity toplevel_tb is
end toplevel_tb;

 
architecture behavior of toplevel_tb is 

component toplevel is  
  port (
    -- PCI FPGA signals
    pciBusy   : in    std_logic;
    pciEmpty  : in    std_logic;
    pciRW     : out   std_logic;
    pciEnable : out   std_logic;
    pciData   : inout std_logic_vector(63 downto 0);

    -- BenERA LEDS
    debugled : out std_logic_vector(15 downto 0);

    clk   : in std_logic;               -- normal (fast) clock
    clk40 : in std_logic;               -- 40MHz clock

    rst : in std_logic);
end component;

  signal pciBusy : std_logic;
  signal pciEmpty : std_logic;
  signal pciRW : std_logic;
  signal pciEnable : std_logic;
  signal pciData : std_logic_vector(63 downto 0);
  
  signal debugled : std_logic_vector(15 downto 0);
  
  signal clk : std_logic;
  signal clk40 : std_logic;
  
  signal rst : std_logic;
  constant clk_period : time := 12.5 ns;

begin

top: toplevel
  port map(
    pciBusy => '0',
    pciEmpty => pciEmpty,
    pciRW => pciRW,
    pciEnable => pciEnable,
    pciData => pciData,
    
    debugled => debugled,
    
    clk => clk,
    clk40 => clk40,
    rst => rst);
    
    
  clk_process :process
   begin
		clk <= '0';
		wait for clk_period/2;
		clk <= '1';
		wait for clk_period/2;
   end process;

  clk40_process :process
   begin
		clk40 <= '0';
		wait for clk_period;
		clk40 <= '1';
		wait for clk_period;
   end process;
   
  rst_process :process
   begin
    rst <='0';
    wait for clk_period;
    rst<= '1';
    wait;
  end process;
end;
