-------------------------------------------------------------------------------
-- Title      : Fitness register
-- Project    : 
-------------------------------------------------------------------------------
-- File       : fitness_reg.vhd
-- Author     : Kjetil Aamodt
-- Company    : 
-- Last update: 2005/04/18
-- Platform   : Spartan-6 LX150T
-------------------------------------------------------------------------------
-- Description: Register for storing of fitness results before sending,
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/30  1.0      aamodt	Created
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

use work.sblock_package.all;

entity fitness_reg is

  port (
    data_out: out std_logic_vector(FITNESS_DATA_BUS_SIZE - 1 downto 0);
    data_in :  in std_logic_vector(FITNESS_DATA_BUS_SIZE - 1 downto 0);

    write_enable: in std_logic;
    read_enable : in std_logic;
    
    stall : in std_logic;
    
    rst : in std_logic;
    clk : in std_logic);

end fitness_reg;

architecture fitness_reg_arch of fitness_reg is

  type fifo_t is array(FITNESS_FIFO_DEPTH - 1 downto 0)
    of std_logic_vector(FITNESS_DATA_BUS_SIZE - 1 downto 0);
  signal fifo : fifo_t;

begin

process(clk, rst)
begin
  if rst = '0' then
    fifo <= (others => (others => '0'));

  elsif rising_edge(clk) then
    if stall = '0' then

       if write_enable = '1' then
         fifo(FITNESS_FIFO_DEPTH - 1 downto 1) <= fifo(FITNESS_FIFO_DEPTH - 2 downto 0);
         fifo(0) <= data_in;
       end if;
      
      if read_enable = '1' then
        data_out <= fifo(FITNESS_FIFO_DEPTH - 1);
        fifo(FITNESS_FIFO_DEPTH - 1 downto 1) <= fifo(FITNESS_FIFO_DEPTH - 2 downto 0);
        fifo(0) <= (others => '0');
      end if;

    end if;
  end if;
end process;




end fitness_reg_arch;
